@@ -1,6 +1,6 @@
 #!/usr/local/bin/perl
 
-# $Id: wsd.pl,v 1.2 2005/04/13 00:00:16 jmichelizzi Exp $
+# $Id: wsd.pl,v 1.13 2008/04/10 04:19:36 tpederse Exp $
 
 use strict;
 use warnings;
@@ -14,7 +14,7 @@ our $mconfig;
 our $contextf;
 our $compfile;
 our $stoplist;
-our $window = 3;
+our $window = 4;
 our $contextScore = 0;
 our $pairScore = 0;
 our $silent;
@@ -52,7 +52,7 @@ if ($help) {
 }
 
 if ($version) {
-    print "wsd.pl version 0.04\n";
+    print "wsd.pl version 0.06\n";
     print "Copyright (C) 2004-2005, Jason Michelizzi and Ted Pedersen\n\n";
     print "This is free software, and you are welcome to redistribute it\n";
     print "under certain conditions.  This software comes with ABSOLUTELY\n";
@@ -68,15 +68,28 @@ unless (defined $contextf) {
 }
 
 unless ($format
-        and (($format eq 'raw') or ($format eq 'parsed') or ($format eq 'tagged'))) {
+        and (($format eq 'raw') or ($format eq 'parsed')
+	     or ($format eq 'tagged') or ($format eq 'wntagged'))) {
     print STDERR "The --format argument is required. This is the type of text to be disambiguated.\n";
     showUsage ();
     exit 1;
 }
 
+unless ($scheme and (($scheme eq 'normal') or ($scheme eq 'random')
+		     or ($scheme eq 'sense1') or ($scheme eq 'fixed'))) {
+    print STDERR "The --scheme argument is required.\n";
+    showUsage ();
+    exit 1;
+}
+
 #my $istagged = isTagged ($contextf);
 my $istagged = $format eq 'tagged' ? 1 : 0;
 
+if ($window < 2) {
+    print STDERR "Error: the window must be 2 or larger!\n\n";
+    exit 1;
+}
+
 unless ($silent) {
     print "Current configuration:\n";
     print "    context file  : $contextf\n";
@@ -84,7 +97,7 @@ unless ($silent) {
     print "    scheme        : $scheme\n";
     print "    tagged text   : ", ($istagged ? "yes" : "no"), "\n";
 
-    if ($scheme eq 'normal') {
+    if (($scheme eq 'normal') or ($scheme eq 'fixed')) {
 	# these items are only relevent to normal mode (not sense1 or random)
 	print "    measure       : $measure\n";
 	print "    window        : ", $window, "\n";
@@ -116,6 +129,7 @@ $options{pairScore} = $pairScore if defined $pairScore;
 $options{contextScore} = $contextScore if defined $contextScore;
 $options{outfile} = $outfile if defined $outfile;
 $options{forcepos} = $forcepos if defined $forcepos;
+$options{wnformat} = 1 if $format eq 'wntagged';
 
 my $sr = WordNet::SenseRelate::AllWords->new (%options);
 
@@ -140,6 +154,7 @@ else {
 }
 
 close FH;
+
 my $i = 0;
 foreach my $sentence (@sentences) {
     my @context = split /\s+/, $sentence;
@@ -247,16 +262,17 @@ sub showUsage
     if ($long) {
 	print "Options:\n";
 	print "\t--context FILE       a file containing the text to be disambiguated\n";
-	print "\t--format FORMAT      type of --context ('raw', 'parsed', or 'tagged')\n";
-	print "\t--scheme SCHEME      disambiguation scheme to use.\n";
-	print "\t                       ('normal', 'sense1', or 'random')\n";
+	print "\t--format FORMAT      type of --context ('raw', 'parsed',\n";
+        print "\t                       'tagged' or 'wntagged')\n";
+	print "\t--scheme SCHEME      disambiguation scheme to use. ('normal', \n";
+	print "\t                       'fixed', 'sense1', or 'random')\n";
 	print "\t--type MEASURE       the relatedness measure to use\n";
 	print "\t--config FILE        a configuration file for the relatedness measure\n";
 	print "\t--compounds FILE     a file of compound words known to WordNet\n";
 	print "\t--stoplist FILE      a file of regular expressions that define\n";
 	print "\t                       the words to be excluded from --context\n";
-	print "\t--window INT         include INT WordNet words to the left\n";  	
-	print "\t                       AND right in the window of context\n"; 
+	print "\t--window INT         window of context will include INT words\n";
+	print "\t                       in all, including the target word.\n";
 	print "\t--contextScore NUM   the  minimum required of a winning score\n";
 	print "\t                       to assign a sense to a target word\n";
 	print "\t--pairScore NUM      the minimum pairwise threshold when\n";
@@ -267,6 +283,7 @@ sub showUsage
 	print "\t--silent             run silently; shows only final output\n";
         print "\t--forcepos           force all words in window of context\n";
         print "\t                       to be same pos as target (pos coercion)\n";
+	print "\t                       are assigned\n";
 	print "\t--help               show this help message\n";
 	print "\t--version            show version information\n";
     }
@@ -276,11 +293,15 @@ __END__
 
 =head1 NAME
 
-wsd.pl - disambiguate words
+wsd.pl - Command line driver for WordNet::SenseRelate::AllWords 
 
 =head1 SYNOPSIS
 
-wsd.pl --context FILE --format FORMAT [--scheme SCHEME] [--type MEASURE] [--config FILE] [--compounds FILE] [--stoplist FILE] [--window INT] [--contextScore NUM] [--pairScore NUM] [--outfile FILE] [--trace INT] [--silent] [--forcepos] | --help | --version
+ wsd.pl --context FILE --format FORMAT [--scheme SCHEME] [--type MEASURE] 
+           [--config FILE] [--compounds FILE] [--stoplist FILE] 
+           [--window INT] [--contextScore NUM] [--pairScore NUM] 
+           [--outfile FILE] [--trace INT] [--silent] [--forcepos] 
+		| --help | --version
 
 =head1 DESCRIPTION
 
@@ -321,14 +342,39 @@ punctuation, such as I<U.S.>, I<Alzheimer's>, I<S/N>, etc.
 Similar to parsed, but the input text has been part-of-speech tagged with
 Penn Treebank tags (perhaps using the Brill tagger).
 
+=item wntagged
+
+Similar to tagged, except that the input should only contain words known to
+WordNet, and each word should have a letter indicating the part of speech
+('n', 'v', 'a', or 'r' for nouns, verbs, adjectives, and adverbs).
+For example:
+
+  dog#n run#v fast#r
+
+Additionally, no attempt will be made to search for other valid forms of the
+words in the input.  For example, if 'dogs#n' occurs in the input, the
+program will not attempt to use other forms such as 'dog#n'.
+
 =back
 
 =item --scheme=B<SCHEME>
 
-The disambiguation scheme to use.  Valid values are "normal", "sense1", 
-and "random". The default is "normal".  WordNet sense 1 disambiguation  
-guesses that the correct sense for each word is the first sense in WordNet  
-because the senses of words in WordNet are ranked according to frequency.   
+The disambiguation scheme to use.  Valid values are "normal", "fixed",
+"sense1", and "random". The default is "normal".  In fixed mode, once a word
+is assigned a sense number, other senses of that word won't be considered
+when disambiguating words to the right of that context word.  For example,
+if the context is
+
+  dogs run very fast
+
+and 'dogs' has been assigned sense number 1, only sense 1 of dogs will
+be used in computing relatedness values when disambiguating 'run', 'very',
+and 'fast'.
+
+WordNet sense 1
+disambiguation  guesses that the correct sense for each word is the
+first sense in WordNet because the senses of words in WordNet are
+ranked according to frequency.   
 The first sense is more likely than the second, the second is more likely  
 than the third, etc. Random selects one of the possible senses of the 
 target word randomly. 
@@ -360,11 +406,18 @@ alternative would be /\ba\b/.
 
 =item --window=B<INTEGER>
 
-Defines the size of the window of context. The default value is 3, which 
-means that 3 words to the left, and 3 words to the right are used by the 
-algorithm. Note that stop words and words not known to WordNet are 
-excluded from the window of context prior to determining the size, so the  
-window is always made up of WordNet words. 
+Defines the size of the window of context.  The default is 4.  A window
+size of N means that there will be a total of N words in the context
+window, including the target word.  If N is a (positive) even number,
+then there will be one more word on the left side of the target word
+than on the right.
+
+For example, if the window size is 4, then there will be two words on
+the left side of the target word and one on the right.  If the window
+is 5, then there will be two words on each side of the target word.
+
+The minimum window size is 2.  A smaller window would mean that there
+were no context words in the window.
 
 =item --contextScore=B<REAL>
 
@@ -392,34 +445,19 @@ Turn tracing on/off.  A value of zero turns tracing off, a non-zero value
 turns tracing on.  The different trace levels can be added together
 to see the combined traces.  The trace levels are:
 
-=over
+  1 Show the context window for each pass through the algorithm.
 
-=item 1
+  2 Display winning score for each pass (i.e., for each target word).
 
-Show the context window for each pass through the algorithm.
+  4 Display the non-zero scores for each sense of each target
+    word (overrides 2).
 
-=item 2
+  8 Display the non-zero values from the semantic relatedness measures.
 
-Display the winning score for each pass (i.e., for each target word). 
+ 16 Show the zero values as well when combined with either 4 or 8.
+    When not used with 4 or 8, this has no effect.
 
-=item 4
-
-Display the non-zero scores for each sense of each target word (overrides 2).
-
-=item 8
-
-Display the non-zero values from the semantic relatedness measures.
-
-=item 16
-
-Show the zero values as well when combined with either 4 or 8.  When not
-used with 4 or 8, this has no effect.
-
-=item 32
-
-Display traces from the semantic relatedness module.
-
-=back
+ 32 Display traces from the semantic relatedness module.
 
 =item --silent
 
@@ -428,26 +466,44 @@ final output.
 
 =item --forcepos
 
-Turn part of speech coercion on.  POS coercion forces other words in the
-context window to be of the same part of speech as the target word.  This
-may be useful when using a measure of semantic similarity that only works
-with noun-noun and verb-verb pairs.
+Turn part of speech coercion on.  POS coercion attempts to force other words
+in the context window to be of the same part of speech as the target word.
+If the text is POS tagged, the POS tags will be ignored.
+POS coercion  may be useful when using a measure of semantic similarity that
+only works with noun-noun and verb-verb pairs.
 
 =back
 
+=head1 SEE ALSO
+
+ L<WordNet::SenseRelate::AllWords>
+
+The main web page for SenseRelate is
+
+ L<http://senserelate.sourceforge.net/>
+
+There are several mailing lists for SenseRelate:
+
+ L<http://lists.sourceforge.net/lists/listinfo/senserelate-users/>
+
+ L<http://lists.sourceforge.net/lists/listinfo/senserelate-news/>
+
+ L<http://lists.sourceforge.net/lists/listinfo/senserelate-developers/>
+
 =head1 AUTHORS
 
-Jason Michelizzi, E<lt>jmichelizzi at users.sourceforge.netE<gt>
+ Jason Michelizzi 
 
-Ted Pedersen, E<lt>tpederse at d.umn.eduE<gt>
+ Ted Pedersen, University of Minnesota, Duluth
+ E<lt>tpederse at d.umn.eduE<gt>
 
 =head1 BUGS
 
-None known.
+Please report to senserelate-users mailing list. 
 
 =head1 COPYRIGHT
 
-Copyright (C) 2004-2005 Jason Michelizzi and Ted Pedersen
+Copyright (C) 2004-2008 Jason Michelizzi and Ted Pedersen
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
